// Copyright 2024 Global Virtual Airlines Group. All Rights Reserved.
package org.deltava.tasks;

import java.io.*;
import java.util.*;
import java.sql.Connection;

import org.apache.logging.log4j.Level;
import org.deltava.beans.acars.ArchiveHelper;
import org.deltava.beans.flight.FlightReport;
import org.deltava.beans.servinfo.PositionData;

import org.deltava.dao.*;
import org.deltava.dao.file.SetSerializedOnline;
import org.deltava.taskman.*;

/**
 * A temporary task to archive orphaned online tracks.
 * @author Luke
 * @version 11.1
 * @since 11.1
 */

public class OnlineArchiveTask extends Task {

	public OnlineArchiveTask() {
		super("Archive Online Track", OnlineArchiveTask.class);
	}

	@Override
	protected void execute(TaskContext ctx) {
		try {
			Connection con = ctx.getConnection();
			
			// Get PIREPs with an online track
			GetOnlineTrack tdao = new GetOnlineTrack(con);
			Collection<Integer> IDs = tdao.getDisposedPIREPs();
			
			// Check if disposed
			GetFlightReports frdao = new GetFlightReports(con);
			for (Integer id : IDs) {
				FlightReport fr = frdao.get(id.intValue(), ctx.getDB());
				if (fr == null) continue;
				
				ctx.startTX();
				
				// Archive the data
				SequencedCollection<PositionData> trackData = tdao.get(fr.getID());
				log.warn("Detected unarchived track data for Flight Report {} - {} entries", id, Integer.valueOf(trackData.size()));
				try (OutputStream os = new BufferedOutputStream(new FileOutputStream(ArchiveHelper.getOnline(fr.getID())))) {
					SetSerializedOnline owdao = new SetSerializedOnline(os);
					owdao.archive(fr.getID(), trackData);
				} catch (IOException ie) {
					throw new DAOException(ie);
				}				
				
				// Purge the track
				SetOnlineTrack twdao = new SetOnlineTrack(con);
				twdao.purge(fr.getID());
				ctx.commitTX();
			}
			
			log.log(IDs.isEmpty() ? Level.INFO : Level.WARN, "{} flights archived", Integer.valueOf(IDs.size()));
		} catch (DAOException de) {
			ctx.rollbackTX();
			logError("Error archiving track", de);
		} finally {
			ctx.release();
		}

		log.info("Completed");
	}
}